#version 330 core

//vertex input
in vec2 fragmentUvs;
smooth in vec3 fragmentNormals;
in vec3 fragmentWorldPosition;
//in uint fragmentMode;

//uniforms from main program
uniform sampler2D	colorTexture;
//uniform mat4	modelView;
uniform int		mode;
//uniform uint mode; //0=3d world and lightning, 1=text (hud, start game and stuff), 3=normals
//in mat4 fragmentInverseModelView;

const int MAX_POINT_LIGHTS = 3;     

struct BaseLight                                                                    
{                                                                                   
    vec3 Color;                                                                     
    float AmbientIntensity;                                                         
    float DiffuseIntensity;                                                         
};                                                                                  
                                                                                    
struct DirectionalLight                                                             
{                                                                                   
    BaseLight Base;                                                                 
    vec3 Direction;                                                                 
};                                                                                  
                                                                                    
struct Attenuation                                                                  
{                                                                                   
    float Constant;                                                                 
    float Linear;                                                                   
    float Exp;                                                                      
};                                                                                  
                                                                                    
struct PointLight                                                                           
{                                                                                           
    BaseLight Base;                                                                         
    vec3 Position;                                                                          
    Attenuation Atten;                                                                      
};    

uniform int numPointLights;      
uniform PointLight gPointLights[MAX_POINT_LIGHTS];  
uniform vec3 gEyeWorldPos;                                                                  
uniform float gMatSpecularIntensity;                                                        
uniform float gSpecularPower;   
uniform vec3 AmbientColor;
uniform float AmbientIntensity;

//pixelcolor
out vec4 color;

vec4 CalcLightInternal(BaseLight Light, vec3 LightDirection, vec3 Normal)                   
{                                                                                           
    vec4 AmbientColor = vec4(Light.Color * Light.AmbientIntensity, 1.0f);
    float DiffuseFactor = dot(Normal, -LightDirection);                                     
                                                                                            
    vec4 DiffuseColor  = vec4(0, 0, 0, 0);                                                  
    vec4 SpecularColor = vec4(0, 0, 0, 0);                                                  
                                                                                            
    if (DiffuseFactor > 0) {                                                                
        DiffuseColor = vec4(Light.Color * Light.DiffuseIntensity * DiffuseFactor, 1.0f);    
                                                                                            
        vec3 VertexToEye = normalize(gEyeWorldPos - fragmentWorldPosition);                             
        vec3 LightReflect = normalize(reflect(LightDirection, Normal));                     
        float SpecularFactor = dot(VertexToEye, LightReflect);                                      
        if (SpecularFactor > 0) {                                                           
            SpecularFactor = pow(SpecularFactor, gSpecularPower);
            SpecularColor = vec4(Light.Color * gMatSpecularIntensity * SpecularFactor, 1.0f);
        }                                                                                   
    }                                                                                       
                                                                                            
    return (AmbientColor + DiffuseColor + SpecularColor);                                   
}                                                                                           
                                                                                            
//vec4 CalcDirectionalLight(vec3 Normal)                                                      
//{                                                                                           
//    return CalcLightInternal(gDirectionalLight.Base, gDirectionalLight.Direction, Normal); 
//}                                                                                           
                                                                                            
vec4 CalcPointLight(int Index, vec3 Normal)                                                 
{                                                                                           
    vec3 LightDirection = fragmentWorldPosition - gPointLights[Index].Position;                         
    float Distance = length(LightDirection);                                                
    LightDirection = normalize(LightDirection);                                             
                                                                                            
    vec4 Color = CalcLightInternal(gPointLights[Index].Base, LightDirection, Normal);       
    float Attenuation =  gPointLights[Index].Atten.Constant +                               
                         gPointLights[Index].Atten.Linear * Distance +                      
                         gPointLights[Index].Atten.Exp * Distance * Distance;               
                                                                                            
    return Color / Attenuation;                                                             
}  

void main()
{ 	
	if (mode == 0)
	{
		vec3 Normal = normalize(fragmentNormals); 
		
		vec4 TotalLight = vec4(AmbientColor,1.0f) * AmbientIntensity;

		for (int i = 0 ; i < numPointLights ; i++) {                                           
			TotalLight += CalcPointLight(i, Normal);                                       
		}                                                                                       
                                                                                            
		color =  texture2D( colorTexture, fragmentUvs ) * TotalLight;
	}
	
	if (mode == 1)
	{
		color = texture2D( colorTexture, fragmentUvs );
	}
	
	if (mode == 2)
	{
		color = vec4 (fragmentNormals,1);
		//normalsShader();
	}
	
}