#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition_modelspace;
layout(location = 1) in vec3 vertexNormals;
layout(location = 2) in vec2 uvs;
layout(location = 3) in vec2 vertexPosition_screenspace;

//output
out vec3 PosL;
out vec2 fragmentUvs;
smooth out vec3 fragmentNormals;
out vec3 fragmentWorldPosition;
//out uint fragmentMode;

//input
//uniform mat4 mvp;
uniform mat4 view;
uniform mat4 projection;
uniform mat4 model;
uniform vec3 lightPosition;
uniform vec3 cameraPosition;

//changing shaders in code is apparently costly
//here's a uniform that tells the shader what to shade
uniform int mode; //0=3d world and lightning, 1=text (hud, start game and stuff), 2=normals
uniform vec2 screenResolution;

void main()
{
	//fragmentMode = mode;

	//for some reason, this if statement throws opengl errors
	if (mode == 0)
	{
		//needed more often
		vec4 vertexPosVec4 = vec4(vertexPosition_modelspace,1);
		PosL = vertexPosition_modelspace;
	
		//faster mvp


		//No Transformation
		//gl_Position = mvp * vertexPosVec4 * vec4(1.0, 1.0 ,1.0, 1.0);
		gl_Position = projection * view * model * vertexPosVec4 * vec4(1.0,1.0,1.0,1.0);

		fragmentWorldPosition = (model * vertexPosVec4).xyz;
		fragmentUvs = uvs;
		fragmentNormals = vec3(model * vec4(vertexNormals,0));
		
	}
	
	if (mode == 1)
	{	
		// Output position of the vertex, in clip space
		// map [0..800][0..600] to [-1..1][-1..1]
		vec2 vertexPosition_homoneneousspace = vertexPosition_screenspace - vec2(400,300); // [0..800][0..600] -> [-400..400][-300..300]
		vertexPosition_homoneneousspace /= vec2(400,300);
		gl_Position =  vec4(vertexPosition_homoneneousspace,0,1);
	
		// UV of the vertex. No special space for this one.
		fragmentUvs = uvs;
	}
	
	if (mode == 2)
	{
		//normalsShader();//shit doesnt work
		//No Transformation
		//gl_Position = mvp * vec4(vertexPosition_modelspace,1);
		gl_Position = projection * view * model * vec4(vertexPosition_modelspace,1);
		//pass incoming normals from model to fragmentshader
		fragmentNormals = vertexNormals;
	}
	
}